// Define the number of retry attempts for FastLED's interrupt handling
#define FASTLED_INTERRUPT_RETRY_COUNT 0

// Include the FastLED library
#include "FastLED.h"

// Define LED parameters
#define LED_TYPE    WS2812B
#define DATA_PIN    4
#define COLOR_ORDER GRB
#define NUM_LEDS    70
#define BRIGHTNESS  80

// Create an array to store LED colors
CRGB leds[NUM_LEDS];

// Define palettes for current and target colors
CRGBPalette16 currentPalette(CRGB::Black);
CRGBPalette16 targetPalette(CRGB::Black);

// Define the number of updates per second for FastLED
#define UPDATES_PER_SECOND 50

// Track the time since the last palette change
unsigned long lastPaletteChangeTime = 0;

// Function to generate a random palette
void generateRandomPalette() {
  // Generate a random number of colors between 3 and 9
  int numColors = random(2, 8);

  // Generate a random base color in the range 0-255
  int baseColor = random(256);

  // Set the golden angle for color variation
  float goldenAngle = 137.5;

  // Print the generated values for debugging
  Serial.print("Number of Colors: "); Serial.print(numColors);
  Serial.print(" | Base Color: "); Serial.println(baseColor);

  // Create the target palette based on the golden angle
  for (int i = 0; i < numColors; i++) {
    int nextColor = int((baseColor + goldenAngle * i) + 256) % 256;
    targetPalette[i] = CHSV(nextColor, 255, 255);
  }
}

// Function to fill LEDs with colors from the palette
void FillLEDsFromPaletteColors(uint8_t colorIndex) {
  uint8_t brightness = 128;

  // Fill LEDs with colors from the current palette
  for (int i = 0; i < NUM_LEDS; i++) {
    leds[i] = ColorFromPalette(currentPalette, colorIndex, brightness);
    colorIndex += 12;
  }
}

// Setup function, executed once at the beginning
void setup() {
  // Add LEDs to FastLED with specified parameters
  delay(1000);
  Serial.begin(115200);
  LEDS.addLeds<LED_TYPE, DATA_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalSMD5050);
  LEDS.setBrightness(BRIGHTNESS);
  Serial.println("FastLED Setup done");

  // Seed the random number generator with analogRead
  randomSeed(analogRead(0));

  // Generate the initial random palette
  generateRandomPalette();
}

// Loop function, executed repeatedly
void loop() {
  // Get the current time
  unsigned long currentTime = millis();

  // Check if it's time to change the palette (every 10 seconds)
  if (currentTime - lastPaletteChangeTime >= 10000) {
    generateRandomPalette();  // Generate a new random palette
    lastPaletteChangeTime = currentTime;  // Update the last change time
    Serial.println("Change palette");
  }

  // Blend the current palette toward the target palette
  uint8_t maxChanges = 24;
  nblendPaletteTowardPalette(currentPalette, targetPalette, maxChanges);

  // Adjust the start index for motion speed
  static uint8_t startIndex = 0;
  startIndex = startIndex + 2; /* motion speed */

  // Fill LEDs with colors from the current palette
  FillLEDsFromPaletteColors(startIndex);

  // Show the colors on the LEDs
  FastLED.show();

  // Delay for the specified updates per second
  FastLED.delay(1000 / UPDATES_PER_SECOND);
}
